wMemDB = {
	numAddons = 75
}
local default = {
	numAddons = 75
}
local ldb = LibStub("LibDataBroker-1.1");
local wM = "|cffe1a500w|cff69ccf0Memory|r";
addonsMem = {}

addons = {};
local numAddons = 0;
local totalMemory = 0;

local function round(num, decPoints)  
   local m = 10^(decPoints or 0);
   return floor(m*num + 0.5)/m;
end

local function OLDmemFormat(mem)
	local memRounded = round(mem, 1)
	if (memRounded > 999) then
		mem = round((memRounded/1024), 1)
		if (mem==math.floor(mem)) then
			mem = (mem..".0")
		end
		mem = (mem.." MB")
	else
		mem = round(mem, 1)
		if (mem==math.floor(mem)) then
			mem = (mem..".0")
		end
		mem = (mem.." KB")
	end
	return mem
end

local function memoryFormat(kilobytes,pieces)
	local rounded = round(kilobytes,1);
	if (rounded > 999) then
		if pieces then return round(rounded/1024,1), "MB" end
		return string.format("%.1f MB",round(rounded/1024,1));
	else
		if pieces then return rounded, "KB" end
		return string.format("%.1f KB",rounded);
	end
end

local function sortFunc(a,b)
	return a.memUsage > b.memUsage;
end

local function sortFunction(a,b)
	return a[2] > b[2];
end

local function updateAddonsData()
	totalMemory = 0;
	UpdateAddOnMemoryUsage()

	numAddons = GetNumAddOns();
	local tableID = 1;
	for curAddonID = 1,numAddons do
		local memoryUsage = GetAddOnMemoryUsage(curAddonID);
		local name, title, _, _, _, _, _ = GetAddOnInfo(curAddonID);
		if (IsAddOnLoaded(curAddonID)) then
			addons[tableID] = {title,memoryUsage};			
			tableID = tableID + 1;
			
			totalMemory = totalMemory + memoryUsage;
		end
	end
	
	sort(addons,sortFunction);
end

local numAddons, addonId, memUsage, name, title, slotNum
wMemoryOptions = CreateFrame("Frame","wMemoryOptions")
local Broker_wMemory = ldb:NewDataObject("Broker_wMemory", {
	type = "data source",
	text = "0 MB",
	value = 0,
	suffix = "MB",
	icon = "interface\\addons\\broker_wmemory\\img\\inv_gizmo_01_transparent.tga",
	label = wM,
	OnClick = function(_, button)
		if (button=="LeftButton" and IsShiftKeyDown()) then
			collectgarbage(collect) print(wM..":|cffffd200 Stinky garbarge removed.");
			updateAddonsData();
		elseif (button=="RightButton") then 
			wMemoryOptions:Show();
		end
	end,
	OnTooltipShow = function(tooltip)
		tooltip:AddLine(wM..":")
		tooltip:AddLine(" ")
		
		updateAddonsData();
		
		for k,v in pairs(addons) do
			if (k<=wMemDB.numAddons or IsShiftKeyDown()) then
				tooltip:AddDoubleLine(v[1], memoryFormat(v[2]),1,1,1,1,1,1);
			end
		end
		tooltip:AddLine(" ")
		tooltip:AddLine("|cff69ccf0Shift-Click|cffffd200 to collect garbage")
		tooltip:AddLine("|cff69ccf0Right-Click|cffffd200 to show options")
	end,
	OnLeave = function(self)
		GameTooltip:Hide()
	end
})

local lastUp = 10
local OnUpdate = CreateFrame("Frame")
OnUpdate:SetScript("OnUpdate", function(self, elap)
	lastUp = lastUp + elap
	if (lastUp>15) then
		
		updateAddonsData();
		Broker_wMemory.text = memoryFormat(totalMemory);
		Broker_wMemory.value, Broker_wMemory.suffix = memoryFormat(totalMemory,true);
		
		lastUp = lastUp - 15
	end
end)

local backdrop = {
  bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
  edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
  edgeSize = 16,
  insets = {left = 4, right = 4, top = 4, bottom = 4,}
}


local f = wMemoryOptions
f:SetBackdrop(backdrop)
f:SetBackdropColor(0,0,0,.9)
f:SetSize(200,85)
f:SetPoint("CENTER",UIParent,"CENTER",0,150)
f.tex = f:CreateTexture("wMemoryOptionsTexture", "OVERLAY")
f.tex:SetSize(40,40)
f.tex:SetTexture("interface\\addons\\broker_wmemory\\img\\inv_gizmo_01_transparent.tga")
f.tex:SetPoint("CENTER",f,"TOPLEFT",15,-15)
f.title = f:CreateFontString("wMemoryOptionsTitle","OVERLAY","ChatFontNormal")
f.title:SetText(wM.." Options")
f.title:SetPoint("TOP",f,"TOP",0,-10)
f.slider = CreateFrame("Slider","wMemoryOptionsSlider",f,"OptionsSliderTemplate")
f.slider:SetPoint("BOTTOM",f,"BOTTOM",0,15)
f.slider:SetSize(150,16)
f.slider:SetMinMaxValues(1,99)
f.slider:SetValue(1)
f.slider:SetValueStep(1)
f.slider:SetOrientation("HORIZONTAL")
_G[f.slider:GetName().."Low"]:SetText("")
_G[f.slider:GetName().."High"]:SetText("")
f.low = f:CreateFontString("wMemoryOptionsLow","OVERLAY","ChatFontSmall")
f.low:SetText("1")
f.low:SetPoint("RIGHT",f.slider,"LEFT",-5,0)
f.high = f:CreateFontString("wMemoryOptionsHigh","OVERLAY","ChatFontSmall")
f.high:SetText("99")
f.high:SetPoint("LEFT",f.slider,"RIGHT",5,0)
f.desc = f:CreateFontString("wMemoryOptionsDescription","OVERLAY","ChatFontSmall")
f.desc:SetText("Display       addon(s) in tooltip.")
f.desc:SetPoint("BOTTOM",f.slider,"TOP",0,10)

local function numberAddonsUpdate(numberEntered)
	if (numberEntered~=nil) and (numberEntered>0) and (numberEntered<100) then
		wMemDB.numAddons = numberEntered
		f.slider:SetValue(wMemDB.numAddons)
		f.num:SetText(wMemDB.numAddons)
		if (numberEntered==1) then f.desc:SetText("Display       addon in tooltip.") else f.desc:SetText("Display       addons in tooltip.") end
	else
		print(wM..": Please enter a number between 0 and 100.")
		f.num:SetText(wMemDB.numAddons)
	end
end

f.num = CreateFrame("EditBox","wMemoryOptionsNumberAddons",f)
f.num:SetSize(30,10)
f.num:SetPoint("LEFT",f.desc,"LEFT",35,-1)
f.num:SetTextInsets(2,0,0,0)
f.num:IsMultiLine(false)
f.num:SetAutoFocus(false)
f.num:SetText("75")
f.num:SetFontObject("GameFontNormalSmall")
f.num:SetMaxBytes(3)
f.num:SetScript("OnEnterPressed", function(self)
    f.num:ClearFocus()
    numberAddonsUpdate(self:GetNumber())
end)
f.slider:SetScript("OnValueChanged",function(self)
	if self == nil then return end
	f.num:SetText(self:GetValue())
end)
f.slider:SetScript("OnMouseUp", function(self)
	if self == nil then return end
	numberAddonsUpdate(self:GetValue())
 end)
f.ver = f:CreateFontString("wMemoryOptionsVersion","OVERLAY","GameFontWhiteTiny")
f.ver:SetPoint("BOTTOMRIGHT",f,"BOTTOMRIGHT",-5,5)
f.ver:SetText("Ver "..GetAddOnMetadata("Broker_wMemory","Version"))
f.ver:SetJustifyH("RIGHT")
f.close = CreateFrame("Button","wMemoryOptionsClose",f,"UIPanelCloseButton")
f.close:SetSize(20,20)
f.close:SetPoint("TOPRIGHT",f,"TOPRIGHT",-2,-2)
f.close:SetScript("OnClick",function(self) f:Hide() end)
f:Hide()

local event = CreateFrame("Frame")
event:SetScript("OnEvent",function(self,event,a1,a2,a3,a4,a5)
	if (event=="ADDON_LOADED") and (a1=="Broker_wMemory") then
		for k,v in pairs(default) do if wMemDB[k] == nil then wMemDB[k] = default[k] end end
		numberAddonsUpdate(wMemDB.numAddons)
	end
end)
event:RegisterEvent("ADDON_LOADED")